#!/usr/bin/python
# -*- coding: utf-8 -*-

import re

import pycurl

from moio.lib.singletonmixin import Singleton

class ConnectionManager(Singleton):
    """Ritorna e configura gli oggetti Curl.
    Questa classe vorrebbe essere un Facade su pycurl, anche se al momento
    non lo è completamente."""
       
    curl = None
    """Istanza dell'oggetto Curl per la comunicazione con le librerie
    libcurl."""
    
    def __init__(self):
        self.curl = pycurl.Curl()
        self.curl.setopt(pycurl.COOKIEFILE, "")
        self.curl.setopt(pycurl.ENCODING, "") #abilita gzip
        self.curl.setopt(pycurl.SSL_VERIFYPEER, 0)# non verifica il certificato SSL
        self.curl.setopt(pycurl.SSL_VERIFYHOST, 0)
        self.curl.setopt(pycurl.FOLLOWLOCATION, True)
   
    def getCurl(self):
        """Ritorna un oggetto Curl (unica istanza nell'applicazione)"""
        return self.curl
    
    def forgetCookiesFromDomain(self, domain):
        """Rimuove i cookie di un certo dominio."""
        cookieList= self.curl.getinfo(pycurl.INFO_COOKIELIST)
        #Toglie tutto
        self.curl.setopt(pycurl.COOKIELIST, "ALL")
        #Rimette il resto
        for i in cookieList:
            if (re.search(domain,i) is None):
                self.curl.setopt(pycurl.COOKIELIST, i)
    
    def isInternetReachable(self):
        """Ritorna True se Internet è raggiungibile."""
        #Pare che al momento non esistano modi migliori...
        return self.ping("http://www.google.it")
    
    def ping(self, url):
        """Ritorna True se il sito all'url specificato è raggiungibile."""
        try:
            c = self.getCurl()
            c.setopt(pycurl.WRITEFUNCTION, self.doNothing)
            c.setopt(pycurl.URL, url)
            c.perform()
        except pycurl.error:
            return False
        return True
    
    def doNothing(self, args):
        """Funzione-parametro per pycurl, scarta la pagina richiesta."""
        pass