#!/usr/bin/python2.4
# -*- coding: utf-8 -*-

import re
import sys
from cStringIO import StringIO

import pycurl

from moio.plugins.Sender import Sender
from moio.plugins.CaptchaDecoder import CaptchaDecoder
from moio.ConnectionManager import ConnectionManager
from moio.errors.SiteCustomError import SiteCustomError
from moio.errors.CaptchaError import CaptchaError
from moio.errors.SiteConnectionError import SiteConnectionError
from moio.errors.SiteAuthError import SiteAuthError
from moio.errors.SenderError import SenderError

class Vodafone(Sender):
    """Permette di spedire SMS dal sito 190.it"""
    
    maxLength = 160
    """Lunghezza massima del messaggio singolo inviabile da questo sito."""

    requiresRegistration = True
    """Questo plugin richiede nome utente e password?"""
    
    def isAvailable(self):
        """Ritorna true se questo plugin è utilizzabile."""
        #Deve essere disponibile un qualunque CaptchaDecoder
        return CaptchaDecoder.getBestPlugin() is not None
    
    def sendOne(self, number, text, username = None, password = None):
        """Spedisce un SMS con soli caratteri ASCII e di lunghezza massima maxLength
        con le credenziali specificate, supponendo Internet raggiungibile.
        """
        try:
            #Costruisco un nuovo oggetto Curl e lo inizializzo
            c = self.connectionManager.getCurl()
            c.setopt(pycurl.POST, False)
            
            if number[:3] == "+39":
                number = number[3:]
            elif number[0]=="+":
                raise SiteCustomError(self.__class__.__name__,
                u"Questo sito permette di inviare SMS solo verso cellulari italiani.")
        
            #Visito la pagina iniziale
            saver = StringIO()
            c.setopt(pycurl.WRITEFUNCTION, saver.write)
            c.setopt(pycurl.URL, "http://www.vodafone.it/190/trilogy/jsp/home.do")
            c.perform()
            self.checkForErrors(saver.getvalue())
                        
            #Sono già autenticato?
            if(re.search("Ciao: <b><!-- TY_DISP -->", saver.getvalue()) is None):
                #No, ammazzo i vecchi cookie e mi riautentico
                self.connectionManager.forgetCookiesFromDomain("190.it")
                self.connectionManager.forgetCookiesFromDomain("vodafone.it")
                saver = StringIO()
                c.setopt(pycurl.WRITEFUNCTION, saver.write)
                c.setopt(pycurl.URL, "https://www.vodafone.it/190/trilogy/jsp/login.do")
                postFields = {}
                postFields["username"] = username
                postFields["password"] = password
                c.setopt(pycurl.POST, True)
                c.setopt(pycurl.POSTFIELDS, self.codingManager.urlEncode(postFields))
                c.perform()

                self.checkForErrors(saver.getvalue())
                if (re.search("Ciao: <b><!-- TY_DISP -->", saver.getvalue()) is None):
                    raise SiteAuthError(self.__class__.__name__)
                                  
            #Visito la pubblicità obbligatoria
            c.setopt(pycurl.POST, False)
            c.setopt(pycurl.WRITEFUNCTION, self.doNothing)         
            c.setopt(pycurl.URL,
                "http://www.vodafone.it/190/trilogy/jsp/dispatcher.do?ty_key=fdt_invia_sms&tk=9616,2")
            c.perform()

            self.checkForErrors(saver.getvalue())

            #Visito il form degli SMS (anche qui obbligatoriamente...)
            c.setopt(pycurl.URL,
                "http://www.areaprivati.vodafone.it/190/trilogy/jsp/dispatcher.do?ty_key=fsms_hp&ipage=next")
            c.perform()

            #Spedisco l'SMS
            saver = StringIO()
            c.setopt(pycurl.WRITEFUNCTION, saver.write)
            postFields = {}
            postFields["pageTypeId"] = "9604"
            postFields["programId"] = "10384"
            postFields["chanelId"] = "-18126"
            postFields["receiverNumber"] = number
            postFields["message"] = text
            c.setopt(pycurl.POST, True)
            c.setopt(pycurl.POSTFIELDS,
                self.codingManager.urlEncode(postFields))
            c.setopt(pycurl.URL,
                "http://www.areaprivati.vodafone.it/190/fsms/prepare.do")
            c.perform()

            self.checkForErrors(saver.getvalue())
            if (re.search(
                "Ti ricordiamo che puoi inviare SMS via Web solo a numeri di cellulare Vodafone",
                  saver.getvalue()) is not None or
                re.search(
                "Il numero di telefono del destinatario del messaggio non e' valido",
                saver.getvalue()) is not None):
                raise SiteCustomError(self.__class__.__name__,
                        u"Questo sito permette di inviare " +
                        u"SMS solo ai cellulari Vodafone.")
            if (re.search("box_sup_limitesms.gif", saver.getvalue()) is not None):
                raise SiteCustomError(self.__class__.__name__,
                                       u"Hai esaurito gli SMS gratis di oggi.")

            captchaBroken = False
            while captchaBroken == False:
                postFields = {}
                if re.search("generateimg.do", saver.getvalue()) is not None:
                    try:
                        saver = StringIO()
                        c.setopt(pycurl.WRITEFUNCTION, saver.write)
                        c.setopt(pycurl.POST, False)
                        c.setopt(pycurl.URL, "http://www.areaprivati.vodafone.it/190/fsms/generateimg.do")
                        c.perform()

                        self.checkForErrors(saver.getvalue())
                        postFields["verifyCode"] = CaptchaDecoder.getBestPlugin().decodeCaptcha(saver, self.__class__.__name__)
                        c.setopt(pycurl.POST, True)
                    except CaptchaError:
                        raise SenderError(self.__class__.__name__)

                postFields["pageTypeId"] = "9604"
                postFields["programId"] = "10384"
                postFields["chanelId"] = "-18126"
                postFields["receiverNumber"] = number
                postFields["message"] = text

                c.setopt(pycurl.POST, True)
                c.setopt(pycurl.POSTFIELDS,
                    self.codingManager.urlEncode(postFields))
        
                #Confermo l'invio
                saver = StringIO()
                c.setopt(pycurl.WRITEFUNCTION, saver.write)
                c.setopt(pycurl.URL,
                    "http://www.areaprivati.vodafone.it/190/fsms/send.do")
                c.perform()

                if (re.search("generateimg.do", saver.getvalue()) is None):
                    captchaBroken = True

            if (re.search("Hai superato il limite giornaliero di SMS",
                saver.getvalue()) is not None):
                raise SiteCustomError(self.__class__.__name__,
                    u"Sono esauriti gli SMS gratuiti di oggi.")
        
            self.checkForErrors(saver.getvalue())
            if (re.search("elaborata correttamente", saver.getvalue()) is None):
                raise SenderError(self.__class__.__name__)
        
        except pycurl.error, e:
            errno, msg = e
            raise SiteConnectionError(self.__class__.__name__, self.codingManager.iso88591ToUnicode(msg))
        
    def checkForErrors(self, page):
        """Solleva un'eccezione se la pagina contiene una segnalazione d'errore dal sito."""
        if(re.search("intervento di manutenzione", page) is not None):
            raise SiteCustomError(self.__class__.__name__, u"Il sito è in manutenzione, riprova più tardi.")
        if(re.search("numero massimo di accessi", page) is not None):
            raise SiteCustomError(self.__class__.__name__, u"Ci sono troppi utenti sul sito, riprova più tardi.")
